@@ -2376,6 +2376,13 @@ ostree_commit_get_parent (GVariant  *commit_variant)
   return ostree_checksum_from_bytes_v (bytes);
 }
 
+/**
+ * ostree_commit_get_timestamp:
+ * @commit_variant: Commit object
+ *
+ * Returns: timestamp in seconds since the Unix epoch, UTC
+ * Since: 2016.3
+ */
 guint64
 ostree_commit_get_timestamp (GVariant  *commit_variant)
 {
@@ -2668,7 +2675,7 @@ _ostree_detached_metadata_append_gpg_sig (GVariant   *existing_metadata,
                                _OSTREE_METADATA_GPGSIGS_NAME,
                                g_variant_builder_end (signature_builder));
 
-  return g_variant_dict_end (&metadata_dict);
+  return g_variant_ref_sink (g_variant_dict_end (&metadata_dict));
 }
 #endif /* OSTREE_DISABLE_GPGME */
 
